// -*- C++ -*-
//
// Class for kernel to process communications
//
// This program is distributed under the terms of the
// Free Software Foundation General Public Licence.
// Copyright Jeremy Fitzhardinge <jeremy@sw.oz.au> 1993

#ifndef __COMM_H_SEEN__
#define __COMM_H_SEEN__

#include <sys/types.h>
#include <sys/time.h>

#pragma interface

class Filesystem;

// Filesystem communications
class Comm
{
	friend Filesystem;

	// List of fd dispatch handers
	struct disp_fd *fdlist;

	// Pointer to kernel fd dispatch handler
	class doreq *req;

	// fd to kernel
	unsigned int	tokern;

	// The filesystem we're part of.
	Filesystem &filesys;

	void delDispatch(struct disp_fd *);
	
public:
	Comm(Filesystem &, unsigned int to, unsigned int from);
	virtual ~Comm();

	struct disp_fd *addDispatch(int fd, class DispatchFD *dfd);
	
	// This doesn't return until there is an IO error or EOF
	int Run();
	int DeferRepl();
	void CloseFDs(int all = 0);
};

// Dispatch class for FDs
class DispatchFD
{
public:
	virtual int dispatch(int fd) = 0;
	virtual ~DispatchFD();
};

#endif /* __COMM_H_SEEN__ */
